package de.superx.elt.util;


import java.util.StringTokenizer;

/**
 * Original von de.memtext.util.GetOpts,
 * für Abwärtskompatibilität mit Kern 4.9/BI 2021.06 
 * 
 */
public class GetOpts {
    private static String[] arguments = null;

    //don't instantiate
    private GetOpts() {

    }

    /**
     * If you pass the arguments you want to analyse to this static helper class
     * using setOpts, you can later use the short form of the methods e.g.
     * isPresent(String option) without having to pass the arguments again.
     * 
     * @param args
     */
    public static void setOpts(String args[]) {
        arguments = args;
    }

    /**
     * checks if the arguments passed before by setOpts contain the given option
     * 
     * @param String
     *            option
     * @return true if arguments contain the option, i.e. one String which
     *         starts with the option-String
     */
    public static boolean isPresent(String option) {
        if (arguments == null)
            throw new IllegalStateException(
                    "must either use setOpts before or call the long version of this method");
        return isPresent(arguments, option);
    }

    /**
     * checks if the arguments contain the given option
     * 
     * @param args
     * @param String
     *            option
     * @return true if arguments contain the option, i.e. one String which
     *         starts with the option-String
     */
    public static boolean isPresent(String args[], String option) {
        boolean result = false;
        for (int i = 0; i < args.length; i++)
            if (args[i] != null && args[i].startsWith(option))
                result = true;
        return result;
    }

    /**
     * Gets a named option from the arguments passed before with setOps. E.g.
     * getOpt("--log") would return the "--log:true" in the arguments
     * 
     * @param option -
     *            name/start of the option
     * @return String whole option
     */
    public static String getOpt(String option) {
        if (arguments == null)
            throw new IllegalStateException(
                    "must either use setOpts before or call the long version of this method");
        return getOpt(option, arguments);
    }

    /**
     * Gets a named option from the arguments. E.g. getOpt("--log") would return
     * the "--log:true" in the arguments
     * 
     * @param option -
     *            name/start of the option
     * @return String whole option
     */
    public static String getOpt(String option, String args[]) {
        if (args == null)
            throw new IllegalStateException("args must not be null");
        String result = null;
        for (int i = 0; i < args.length; i++)
            if (args[i].startsWith(option))
                result = args[i];
        if (result == null)
            throw new RuntimeException("Option " + option + " not found!");
        return result;
    }

    /**
     * Gets the value of an option from the arguments passed before with setOpts
     * if for example, you ask getValue("--log:") and the arguments passed
     * before with setOpts contain a String "--log:true") "true" is returned
     * 
     * @param String
     *            option
     * @return String value of the option
     */
    public static String getValue(String option) {
        if (arguments == null)
            throw new IllegalStateException(
                    "must either use setOpts before or call the long version of this method");
        return getValue(option, arguments);
    }

    /**
     * Gets the value of an option, if for example, you ask getValue("--log:")
     * and the arguments passed before with setOpts contain a String
     * "--log:true") "true" is returned
     * 
     * @param String
     *            option
     * @return String value of the option
     */
    public static String getValue(String option, String args[]) {
        String result = "";
        String raw = getOpt(option, args);
        int pos = option.length();
        if (raw.charAt(pos) == ':')
            pos++;
        result = raw.substring(pos, raw.length());
        return result;
    }

    /**
     * Prüft, ob alle notwendigen Optionen angegeben sind, z.B. String
     * "-loggingProperties:,-dbProperties" übergeben, die beiden müssen dabei
     * sein
     * 
     * @param options
     *            z.B. "-loggingProperties:,-dbProperties,..."
     * @return null - alles OK, ansonsten String mit den fehlenden Optionen
     */
    public static String isAllRequiredOptionsPresent(String options) {
        String result = null;
        StringTokenizer st = new StringTokenizer(options, ",");
        for (; st.hasMoreTokens();) {
            String optionName = st.nextToken();
            if (!isPresent(optionName)) {
                if (result == null)
                    result = optionName;
                else
                    result += "," + optionName;
            }
        }
        return result;
    }
}
